/*
 * =====================================================================================
 *
 *       Filename:  input.h
 *
 *    Description:  Input class (keyboard / mouse)
 *
 *        Version:  1.0
 *        Created:  12.05.2016 21:25:47
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Simon Ferber (https://www.ist-schlau.de), simon@ist-schlau.de
 *   Organization:  Kiwi Homebrew Computer
 *
 * =====================================================================================
 */

#ifndef INPUT_H
#define INPUT_H

#include "screen.h"
#include "mc68681.h"
#include <cstdint>

/* scancode definitions */
#define KEY_RELEASED 0x80       /* This bit set, when key-release scancode */

#define KEY_LSHIFT  0x2a
#define KEY_RSHIFT  0x36
#define KEY_CTRL    0x1d
#define KEY_ALT     0x38
#define KEY_CAPS    0x3a

/*
 * These bit flags are set in the "shifty" byte based on the state
 * of control keys on the keyboard, like:
 * right shift, left shift, control, alt, ...
 */
#define MODE_RSHIFT 0x01        /* Right Shift key is down */
#define MODE_LSHIFT 0x02        /* Left Shift key is down  */
#define MODE_CTRL   0x04        /* Control is down         */
#define MODE_ALT    0x08        /* Alternate is down       */
#define MODE_CAPS   0x10        /* CapsLock is down        */
#define MODE_CLEAR  0x20        /* Clr/Home is down        */

#define MODE_SHIFT  (MODE_RSHIFT|MODE_LSHIFT)   /* shifted */
#define MODE_SCA    (MODE_RSHIFT|MODE_LSHIFT|MODE_CTRL|MODE_ALT)

#define DEADMIN 1
#define DEADMAX 7
#define DEAD(i) (i + DEADMIN)

class input
{
public:
    struct mousepos
    {
        uint16_t x;
        uint16_t y;
    } mousepos;
    input();

    unsigned char rs232getchar();
    //unsigned char kbgetchar();
    uint32_t kbgetchar();
    unsigned char msgetchar();
    uint16_t rs232getbuffcnt();
    uint8_t kbgetbuffcnt();
    uint8_t msgetbuffcnt();

    void MouseAction (screen *myScreen);
    void showMouse();
    void hideMouse();
    void dispatch();

    void setRS232();

    static void put_rs232buff (unsigned char c);
    static void put_mskbbuff (unsigned char c);
    void decode_scancode (uint8_t scancode);
private:
    uint8_t shifty;
    uint8_t kb_ticks;
    uint16_t kb_initial;
    uint16_t kb_repeat;
    int kb_dead;
    int kb_altnum;
    long kb_last_key;
    struct keytbl {
        /* 128-sized array giving char codes for each scan code */
        const uint8_t *norm;
        const uint8_t *shft;
        const uint8_t *caps;
        /* couples of (scan code, char code), ended by byte zero */
        const uint8_t *altnorm;
        const uint8_t *altshft;
        const uint8_t *altcaps;
        /* table of at most eight dead key translation tables */
        const uint8_t * const *dead;
    };
    struct keytbl current_keytbl;

    //void push_ikbdiorec(uint32_t value);

    /* rs232 buffer */
    static unsigned char rs232buffer[RS232_BUFF_SIZE];
    static volatile unsigned char *rs232inpt, *rs232outpt; // Ringbuffer rs232
    static volatile uint16_t rs232buffcnt;

    /* Mouse/Keyboard buffer */
    static unsigned char mskbbuffer[BUFF_SIZE];
    static volatile unsigned char *mskbinpt, *mskboutpt; // Ringbuffer mouse/keyboard
    static volatile unsigned char mskbbuffcnt;

    /* Keyboard buffer */
    //unsigned char kbbuffer[BUFF_SIZE];
    //volatile unsigned char *kbinpt, *kboutpt; // Ringbuffer keyboard
    uint32_t kbbuffer[BUFF_SIZE];
    volatile uint32_t *kbinpt, *kboutpt; // Ringbuffer keyboard
    volatile unsigned char kbbuffcnt;

    /* Mouse buffer */
    unsigned char msbuffer[BUFF_SIZE];
    volatile unsigned char *msinpt, *msoutpt; // Ringbuffer mouse
    volatile unsigned char msbuffcnt;

    int16_t x=0, y=0, deltax=0, deltay=0;
    uint8_t high_x=0, low_x=0, high_y=0, low_y=0;

    //void put_kbbuff (unsigned char c);
    void put_kbbuff (uint32_t c);
    void put_msbuff (unsigned char c);
    unsigned char mskbgetchar();
    void loadMousepointer(); // Load mousepointer gfx to VRAM
    uint32_t kbshift(uint16_t flag);
};


#endif
